@@ -12,9 +12,9 @@ use Zim::Components::PageView;
 
 # TODO make components configable/autoloading like plugins
 
-our $VERSION = 0.08;
+our $VERSION = '0.10';
 our $LONG_VERSION = << "EOT";
-zim $VERSION
+zim $VERSION - A desktop wiki
 
 Copyright (c) 2005 Jaap G Karssenberg. All rights reserved.
 This program is free software; you can redistribute it and/or
@@ -74,23 +74,132 @@ our $LINK_ICON = Gtk2->CHECK_VERSION(2, 6, 0) ? 'gtk-connect' : 'gtk-convert';
 our %CONFIG = (  # Default config values
 	pane_pos	=> 120,
 	pane_vis	=> 0,
-	pathbar_type	=> 'trace',
-	icon_size	=> 'large-toolbar',
+	statusbar_vis	=> 1,
+	toolbar_vis	=> 1,
+	pathbar_type	=> 'recent',
+	#icon_size	=> 'large-toolbar',
 	width		=> 500,
 	height		=> 350,
 	default_root	=> undef,
 	default_home	=> 'Home',
 	browser		=> undef,
+	file_browser	=> undef,
 	date_string	=> '%A %d/%m/%Y',
 	date_page	=> ':Date:%Y_%m_%d',
 	hist_max	=> 20,
+	recent_show	=> 7,
 	undo_max	=> 50,
 	follow_new_link => 1,
 	use_xdg_cache	=> 0,
 	use_tray_icon	=> 0,
+	use_gtkspell	=> 0,
+	show_spell      => 1,
 	backsp_unindent => 1,
 );
 
+my @ui_actions = (
+	# name,          stock id,  label
+	[ 'FileMenu',    undef,     '_File'         ],
+	[ 'EditMenu',    undef,     '_Edit'         ],
+	[ 'ViewMenu',    undef,     '_View'         ],
+	[ 'InsertMenu',  undef,     '_Insert'       ],
+	[ 'FormatMenu',  undef,     'For_mat'       ],
+	[ 'ToolsMenu',   undef,     '_Tools'        ],
+	[ 'GoMenu',      undef,     '_Go'           ],
+	[ 'HelpMenu',    undef,     '_Help'         ],
+	[ 'PathBarMenu', undef,     'P_athbar type' ],
+	# name,       stock id,         label,          accelerator,   tooltip
+	[ 'Save',     'gtk-save',       '_Save',        '<ctrl>S',     'Save page',    \&on_Save     ],
+	[ 'Close',    'gtk-close',      '_Close',       '<ctrl>W',     'Close window', \&on_Close    ],
+	[ 'Quit',     'gtk-quit',       '_Quit',        '<ctrl>Q',     'Quit',         \&on_Quit     ],
+	[ 'Reload',   'gtk-refresh',    '_Reload',      '<ctrl>R',     'Reload',       \&on_Reload   ],
+	[ 'OpenDir',  'gtk-open',       'Open _folder', undef,         'Open folder',  \&on_OpenDir  ],
+	[ 'Back',     'gtk-go-back',    '_Back',        '<alt>Left',   'Go back',      \&on_Back     ],
+	[ 'Forward',  'gtk-go-forward', '_Forward',     '<alt>Right',  'Go forward',   \&on_Forward  ],
+	[ 'RBack',    undef,            '_Back',        '<alt>Page_Down', undef,       \&on_RBack    ],
+	[ 'RForward', undef,            '_Forward',     '<alt>Page_Up',   undef,       \&on_RForward ],
+	[ 'Parent',   'gtk-go-up',      '_Parent',      '<alt>Up',     'Go to parent', \&on_Parent   ],
+	[ 'Child',    'gtk-go-down',    '_Child',       '<alt>Down',   'Go to child',  \&on_Child    ],
+	[ 'Today',    undef,            'To_day',       '<alt>D',      'Today',        \&on_Today    ],
+	[ 'Home',     'gtk-home',       '_Home',        '<alt>Home',   'Go home',      \&on_Home     ],
+	[ 'JumpTo',   'gtk-jump-to',    '_Jump to...',  '<ctrl>J',     'Jump to page', \&on_JumpTo   ],
+	[ 'Help',     'gtk-help',       '_Help',        'F1',          'Help',         \&on_Help     ],
+	[ 'About',    'gtk-about',      '_About',       undef,         'About',        \&on_About    ],
+);
+my @ui_toggle_actions = (
+	# name,         stock id,    label,        accelerator, tooltip 
+	[ 'TToolBar',   undef,       '_Toolbar',   undef,       'Show toolbar',   \&on_TToolBar,   0],
+	[ 'TStatusBar', undef,       '_Statusbar', undef,       'Show statusbar', \&on_TStatusBar, 0],
+	[ 'TPane',      'gtk-index', 'Side _Pane', 'F9',        'Show side pane', \&on_TPane,      0],
+);
+my @ui_radio_actions = (
+	# name,          stock id, label,           accelerator, tooltip, value
+	[ 'PBRecent',    undef,    '_Recent pages', undef,       '',      0],
+	[ 'PBHistory',   undef,    '_History',      undef,       '',      1],
+	[ 'PBNamespace', undef,    '_Namespace',    undef,       '',      2],
+	[ 'PBHidden',    undef,    'H_idden',       undef,       '',      3],
+);
+my %pathbar_map = ( 'recent' => 0, 'history' => 1, 'namespace' => 2, 'hidden' => 3);
+
+my $ui_layout = q{<ui>
+	<menubar name='MenuBar'>
+		<menu action='FileMenu'>
+			<menuitem action='Save'/>
+			<separator/>
+			<menuitem action='Close'/>
+			<menuitem action='Quit'/>
+		</menu>
+		<menu action='EditMenu'></menu>
+		<menu action='ViewMenu'>
+			<menuitem action='TToolBar'/>
+			<menuitem action='TStatusBar'/>
+			<menuitem action='TPane'/>
+			<menu action='PathBarMenu'>
+				<menuitem action='PBRecent'/>
+				<menuitem action='PBHistory'/>
+				<menuitem action='PBNamespace'/>
+				<menuitem action='PBHidden'/>
+			</menu>
+			<separator/>
+			<menuitem action='Reload'/>
+		</menu>
+		<menu action='InsertMenu'></menu>
+		<menu action='FormatMenu'></menu>
+		<menu action='ToolsMenu'>
+			<menuitem action='OpenDir'/>
+		</menu>
+		<placeholder name='PluginMenus'></placeholder>
+		<menu action='GoMenu'>
+			<menuitem action='Back'/>
+			<menuitem action='Forward'/>
+			<menuitem action='Parent'/>
+			<separator/>
+			<menuitem action='Home'/>
+			<menuitem action='Today'/>
+			<menuitem action='JumpTo'/>
+		</menu>
+		<menu action='HelpMenu'>
+			<menuitem action='Help'/>
+			<menuitem action='About'/>
+		</menu>
+	</menubar>
+	<toolbar name='ToolBar'>
+		<placeholder name='View'>
+			<toolitem action='TPane'/>
+		</placeholder>
+		<separator/>
+		<placeholder name='Go'>
+			<toolitem action='Home'/>
+			<toolitem action='Back'/>
+			<toolitem action='Forward'/>
+		</placeholder>
+		<separator/>
+	</toolbar>
+	<accelerator action='Child'/>
+	<accelerator action='RBack'/>
+	<accelerator action='RForward'/>
+</ui>};
+
 =item C<new(SETTINGS)>
 
 Simple constructor.
@@ -106,6 +215,10 @@ sub new {
 	my $self = bless {settings => $settings}, $class;
 	$self->load_config;
 
+	$self->{settings}{pathbar_type} = 'recent'
+		if $self->{settings}{pathbar_type} eq 'trace';
+		# reset 0.08 default value
+
 	$self->{_message_timeout} = -1;
 	$self->{_save_timeout} = -1;
 	
@@ -155,57 +268,56 @@ sub gui_init {
 	$window->set_title('Zim');
 	$self->{window} = $window;
 
-	my $accels = Gtk2::AccelGroup->new;
-	$window->add_accel_group($accels);
-	$self->{accels} = $accels;
-
-	my $vbox = Gtk2::VBox->new(0, 3);
+	my $vbox = Gtk2::VBox->new(0, 0);
 	$window->add($vbox);
 	$self->{vbox} = $vbox;
 
-	my $toolbar = Gtk2::Toolbar->new();
-	$toolbar->set_style('icons');
-	$vbox->pack_start($toolbar, 0,1,0);
-	$self->{toolbar} = $toolbar;
+	my $actions = Gtk2::ActionGroup->new("Actions");
+	$actions->add_actions(\@ui_actions, $self);
+	$actions->add_toggle_actions(\@ui_toggle_actions, $self);
+	$actions->add_radio_actions(\@ui_radio_actions,
+		$pathbar_map{$self->{settings}{pathbar_type}}, \&on_TPathBar, $self);
+	$actions->get_action('Save')->set_sensitive(0) if $self->{settings}{read_only};
+	$actions->get_action('Back')->set_sensitive(0);
+	$actions->get_action('Forward')->set_sensitive(0);
+	$actions->get_action('Parent')->set_sensitive(0);
+	$self->{actions} = $actions;
+	
+	my $ui = Gtk2::UIManager->new;
+	$ui->insert_action_group($actions, 0);
+	$window->add_accel_group($ui->get_accel_group);
+	$ui->add_ui_from_string($ui_layout);
+	$self->{menubar} = $ui->get_widget("/MenuBar");
+	$vbox->pack_start($self->{menubar}, 0,1,0);
+	$self->{toolbar} = $ui->get_widget("/ToolBar");
+	$vbox->pack_start($self->{toolbar}, 0,1,0);
+	$self->{toolbar}->hide;
+	$self->{toolbar}->set_no_show_all(1);
+	$self->{ui} = $ui;
 
 	my $hpaned = Gtk2::HPaned->new();
 	$hpaned->set_position($self->{settings}{pane_pos});
-	$vbox->add($hpaned);
+	$vbox->pack_start($hpaned, 1,1,0);
 	$self->{hpaned} = $hpaned;
 
-	my $r_vbox = Gtk2::VBox->new(0, 5);
+	my $r_vbox = Gtk2::VBox->new(0, 0);
 	$hpaned->add2($r_vbox);
 	$self->{r_vbox} = $r_vbox;
 
 	my $statusbar = Gtk2::Statusbar->new;
+	$statusbar->set_no_show_all(1);
 	$vbox->pack_start($statusbar, 0,1,0);
 	$self->{statusbar} = $statusbar;
-
-	$self->add_button(
-		'Toggle Index', 'Alt I', 'gtk-index',
-		sub {$self->toggle_pane($self->{settings}{pane_vis} ? 0 : 1)} );
 	
 	my $tree_view = Zim::Components::TreeView->new(app => $self);
 	$tree_view->signal_connect(row_activated =>
 		sub { $self->toggle_pane(0) if $self->{_pane_visible} == -1 } );
+	$tree_view->widget->set_no_show_all(1);
 	$hpaned->add1($tree_view->widget);
 	$self->{objects}{TreeView} = $tree_view;
 
-	$toolbar->append_space;
-	
-	$self->add_button('Home', 'Alt Home', 'gtk-home',
-		sub {$self->load_page($self->{settings}{home})} );
-	my $back_button = $self->add_button(
-		'Go Back', 'Alt Left', 'gtk-go-back', sub {$self->go_back(1)} );
-	my $forw_button = $self->add_button(
-		'Go Forward', 'Alt Right', 'gtk-go-forward', sub {$self->go_forw(1)} );
-	$back_button->set_sensitive(0);
-	$forw_button->set_sensitive(0);
-
-	$toolbar->append_space;
-
 	my $path_bar = Zim::Components::PathBar->new(app => $self);
-	$r_vbox->pack_start($path_bar->widget, 0,1,0);
+	$r_vbox->pack_start($path_bar->widget, 0,1,5);
 	$self->{objects}{PathBar} = $path_bar;
 
 	my $page_view = Zim::Components::PageView->new(app => $self);
@@ -224,52 +336,28 @@ sub gui_init {
 		$self->{settings}{hist_file} ||= File::Spec->catfile($root, '.zim.history'); 
 	}
 	my $history = Zim::History->new(
-		$self->{settings}{hist_file}, $page_view, $self->{settings}{hist_max});
+		$self->{settings}{hist_file}, $page_view,
+		$self->{settings}{hist_max} );
 	$self->{objects}{History} = $history;
 
-	$toolbar->append_space;
-
-	$self->add_button('Actions', '', 'gtk-execute',
-		sub {$self->filechooser_dialog} );
-	$self->add_button('About', 'F1', 'gtk-help',
-		sub {$self->about_dialog} );
-
-
-	$self->add_key('Ctrl S', sub {$self->save_page('FORCE')})
-		unless $self->{settings}{read_only};
-
-	$self->add_key('Ctrl G', sub {$self->goto_page_dialog});
-	$self->add_key('Ctrl R', sub {$self->reload});
-	$self->add_key('Ctrl Q', sub {$self->quit('FORCE')});
-	$self->add_key('Ctrl W', sub {$self->quit});
-	$self->add_key('Alt Up', sub {
-		my $namespace = $self->{page}->namespace;
-		return if $namespace eq ':';
-		$self->load_page($namespace);
-	} );
-	$self->add_key('Alt Down', sub {
-		my $namespace = $self->History->get_namespace;
-		my $name = $self->{page}->name;
-		return unless $namespace =~ /^(:*$name:+[^:]+)/;
-		$self->load_page($1);
-	} );
-	$self->add_key('Alt D', sub {
-		my $page = strftime($self->{settings}{date_page}, localtime);
-		$self->load_page($page);
-	} );
-	$self->add_key('Ctrl  ', sub { # Ctrl-Space
-		if ($self->{settings}{pane_vis}) {
-			if ($tree_view->has_focus) { $page_view->grab_focus }
-			else                       { $tree_view->grab_focus }
-		}
-		else { $self->toggle_pane($self->{_pane_visible} ? 0 : -1) }
-	} );
-	$self->add_key('Alt J', sub {$self->toggle_path_bar} );
-
+	# Some wiring
+	$ui->get_accel_group->connect( # Ctrl-Space
+			ord(' '), ['control-mask'], ['visible'],
+			sub {
+				if ($self->{settings}{pane_vis}) {
+				if ($tree_view->has_focus) { $page_view->grab_focus }
+				else                       { $tree_view->grab_focus }
+			}
+			else { $self->toggle_pane($self->{_pane_visible} ? 0 : -1) }
+		} );
+	
 	$self->signal_connect('page_loaded', sub {
-		my ($back, $forw) = $history->state;
-		$back_button->set_sensitive($back?1:0);
-		$forw_button->set_sensitive($forw?1:0);
+		my $self = shift;
+		my $state = $self->History->get_state;
+		$self->{actions}->get_action('Back')->set_sensitive($$state{back} ? 1:0);
+		$self->{actions}->get_action('Forward')->set_sensitive($$state{forw} ? 1:0);
+		my $namespace = ($self->{page}->namespace =~ /[^:]/) ? 1 : 0;
+		$self->{actions}->get_action('Parent')->set_sensitive($namespace);
 	} );
 
 	# set save interval
@@ -278,13 +366,29 @@ sub gui_init {
 		unless $self->{settings}{read_only};
 	
 	$window->show_all;
-
+	$statusbar->set_no_show_all(0);
+	$tree_view->widget->set_no_show_all(0);
+	$self->{toolbar}->set_no_show_all(0);
+	
 	if ($self->{settings}{use_tray_icon}) {
 		eval {
 			eval 'use Zim::Components::TrayIcon'; die $@ if $@;
 			my $trayicon = Zim::Components::TrayIcon->new(app => $self);
+			$self->{objects}{TrayIcon} = $trayicon;
 		} ;
-		warn $@ if $@;
+		$self->error_dialog(
+			"Failed to load TrayIcon,\n".
+			"do you have Gtk2::TrayIcon installed ?", $@) if $@;
+	}
+	if ($self->{settings}{use_gtkspell}) {
+		eval {
+			eval 'use Zim::Components::Spell'; die $@ if $@;
+			my $spell = Zim::Components::Spell->new(app => $self);
+			$self->{objects}{Spell} = $spell;
+		} ;
+		$self->error_dialog(
+			"Failed to load spell checking,\n".
+			"do you have Gtk2::Spell installed ?", $@) if $@;
 	}
 
 #	unless ($self->{settings}{read_only}) {
@@ -301,13 +405,116 @@ sub gui_init {
 #		$self->{lock_file} = $lock_file;
 #	}
 	
-	# Try saving when the user exits shell or desktop
+	# Try saving the program on system signals
 	$SIG{$_} = sub {
+		print "Signal $_ caught\n";
 		$self->quit('FORCE');
 		exit 1;
-	} for qw/INT TERM HUP/;
+	} for qw/TERM HUP PIPE/;
+
+	# Try saving on desktop exit
+	$window->get_display->signal_connect( closed => sub {
+			print "Display closed\n";
+		} );
 }
 
+sub on_Save { pop->save_page('FORCE') }
+
+sub on_Close { pop->quit() }
+
+sub on_Quit { pop->quit('FORCE') }
+
+sub on_Reload { pop->reload }
+
+sub on_OpenDir {
+	my $self = pop;
+	my $dir = $self->{page}->properties->{base};
+	return unless length $dir;
+	#warn "Opening $dir\n";
+	$self->link_clicked($dir);
+}
+
+sub on_TToolBar {
+	my $self = pop;
+	my $vis = $self->{toolbar}->visible ? 0 : 1;
+	$vis ? $self->{toolbar}->show : $self->{toolbar}->hide ;
+	$self->{settings}{toolbar_vis} = $vis;
+}
+
+sub on_TStatusBar {
+	my $self = pop;
+	my $vis = $self->{statusbar}->visible ? 0 : 1;
+	$vis ? $self->{statusbar}->show : $self->{statusbar}->hide ;
+	$self->{settings}{statusbar_vis} = $vis;
+}
+
+sub on_TPane {
+	my $self = pop;
+	my $vis = $self->{objects}{TreeView}->widget->visible ? 0 : 1;
+	$self->toggle_pane($vis)
+}
+
+sub on_TPathBar {
+	my $self = pop;
+	my $type = lc pop->get_name;
+	$type =~ s/^pb//;
+	$self->toggle_path_bar($type);
+}
+
+sub on_Back { pop->go_back(1) }
+
+sub on_Forward { pop->go_forw(1) }
+
+sub on_RBack {
+	my $self = pop;
+	my ($i, @recent) = $self->History->get_recent;
+	return unless $i > 0;
+	my $rec = $recent[$i-1];
+	$self->load_page($rec);
+}
+
+sub on_RForward {
+	my $self = pop;
+	my ($i, @recent) = $self->History->get_recent;
+	return unless $i < $#recent;
+	my $rec = $recent[$i+1];
+	$self->load_page($rec);
+}
+
+sub on_Parent {
+	my $self = pop;
+	my $namespace = $self->{page}->namespace;
+	return if $namespace eq ':';
+	$self->load_page($namespace);
+}
+
+sub on_Child {
+	my $self = pop;
+	my $namespace = $self->History->get_namespace;
+	my $name = $self->{page}->name;
+	return unless $namespace =~ /^(:*$name:+[^:]+)/;
+	$self->load_page($1);
+}
+
+sub on_Home {
+	my $self = pop;
+	my $home = $self->{repository}->resolve_page($self->{settings}{home});
+	$self->load_page($home);
+}
+
+sub on_Today {
+	my $self = pop;
+	my $page = strftime($self->{settings}{date_page}, localtime);
+	$page = $self->{repository}->resolve_page($page);
+	$self->load_page($page);
+}
+
+sub on_JumpTo { pop->jump_page_dialog }
+
+sub on_Help { pop->show_help }
+
+sub on_About { pop->about_dialog }
+
 =item C<widget()>
 
 Returns the root window widget.
@@ -417,6 +624,8 @@ sub main_loop {
 	unless ($self->{page}) {
 		my $page = $self->{objects}{History}->get_current
 			|| $self->{settings}{home};
+		$page = $self->{repository}->resolve_page($page)
+			unless ref $page;
 		$self->load_page($page);
 		# FIXME what if we are read_only and page does not exist ?
 		# 	load_apge returns undef
@@ -425,7 +634,11 @@ sub main_loop {
 	}
 	
 	$self->{message_lock} = 1;
-	$self->toggle_pane($self->{settings}{pane_vis});
+	$self->{objects}{TreeView}->load_index();
+	my ($actions, $settings) = ($self->{actions}, $self->{settings});
+	$actions->get_action('TToolBar')->set_active($settings->{toolbar_vis});
+	$actions->get_action('TStatusBar')->set_active($settings->{statusbar_vis});
+	$actions->get_action('TPane')->set_active($settings->{pane_vis});
 	$self->toggle_path_bar($self->{settings}{pathbar_type});
 	$self->{message_lock} = 0;
 
@@ -442,8 +655,6 @@ Exit the main loop.
 
 sub quit {
 	my $self = shift;
-	return if $$self{_saved_state};
-	$$self{_saved_state} = 1;
 	$self->on_delete_event(@_) && return;
 	Gtk2->main_quit;
 }
@@ -496,15 +707,19 @@ sub link_clicked {
 	return warn "Warning: You tried to folow an empty link.\n"
 		unless length $link;
 
-	if ($link =~ /^(\w+:\/\/|mailto:)/) { # link is an url
-		my $browser = $self->{settings}{browser} || $ENV{BROWSER};
+	if ($link =~ /^(?:(\w+):\/\/|mailto:)/) { # link is an url
+		my $type = ($1 eq 'file') ? 'file_browser' : 'browser';
+		my $browser = $self->{settings}{$type};
+		$browser ||= $ENV{BROWSER} if $type eq 'browser';
+		$link =~ s#^file://## if $type eq 'file_browser';
 		unless ($browser) {
-			$browser = $self->prompt_browser_dialog;
+			$browser = $self->prompt_browser_dialog($type);
 			return $self->error_dialog('You have no browser configured')
 				unless $browser;
-			$self->{settings}{browser} = $browser;
+			$self->{settings}{$type} = $browser;
 		}
 		$browser =~ s/\%s/$link/ or $browser .= ' '.$link;
+		#warn "Executing: $browser\n";
 		unless (fork) { # child process
 			exec $browser;
 			exit 1; # just to be sure
@@ -525,9 +740,8 @@ Go back one or more steps in the history stack.
 sub go_back {
 	my $self = shift;
 	my $i = shift || 1;
-	my $hist = $self->{objects}{History};
-	$hist->back($i) or return;
-	$self->load_page($hist->get_current);
+	my $rec = $self->{objects}{History}->back($i) || return;
+	$self->load_page($rec);
 }
 
 =item C<go_forw(INT)>
@@ -539,9 +753,8 @@ Go forward one or more steps in the history stack.
 sub go_forw {
 	my $self = shift;
 	my $i = shift || 1;
-	my $hist = $self->{objects}{History};
-	$hist->forw($i) or return;
-	$self->load_page($hist->get_current);
+	my $rec = $self->{objects}{History}->forw($i) || return;
+	$self->load_page($rec);
 }
 
 =item C<reload()>
@@ -552,8 +765,8 @@ Save and reload the current page.
 
 sub reload {
 	my $self = shift;
-	my $hist = $self->{objects}{History};
-	$self->load_page($hist->get_current);
+	$self->load_page(
+		$self->{objects}{History}->get_current );
 }
 
 =item C<load_page(PAGE)>
@@ -562,6 +775,8 @@ Loads a new page in the PageView, updates history etc. when
 necessary. PAGE should be either an absolute page name, a history record
 or a page object.
 
+use C<link_clicked()> for relative page names, urls etc.
+
 =cut
 
 sub load_page {
@@ -570,12 +785,16 @@ sub load_page {
 	if ($self->{page}) {
 		$self->{save_lock} = 0; # just to be sure
 		$self->save_page || return;
+		$self->{repository}->close_page($self->{page});
 	}
 
 	my $rec;
 	if (ref($page) eq 'HASH') { # hist record
 		($rec, $page) = ($page, $page->{name});
 	}
+	else {
+		$rec = $self->{objects}{History}->get_record($page);
+	}
 	
 	if (ref $page) { # page object
 		$self->{page} = $page;
@@ -586,13 +805,13 @@ sub load_page {
 			unless length $page;
 		
 		my $p;
-		eval { $p = $self->{repository}->load_page($page, @args) };
+		eval { $p = $self->{repository}->open_page($page, @args) };
 		return $self->error_dialog("Could not load page $page\n\n$@")
 			if $@;
 		
 		return $self->error_dialog("Page does not exist:\n$page")
-			if $self->{settings}{read_only}
-			and $p->status eq 'new';
+			unless defined $p
+			and not ($self->{settings}{read_only} and $p->status eq 'new');
 
 		$self->{page} = $p;
 	}
@@ -604,7 +823,7 @@ sub load_page {
 		# TODO handle more gratiously
 		# .. switching read_only would help here
 	}
-	$rec = $self->{objects}{History}->record($page) unless defined $rec;
+	
 	$self->{objects}{PageView}->set_state(%$rec) if defined $rec;
 	$self->signal_emit('page_loaded', $self->{page}->name);
 	
@@ -638,18 +857,17 @@ sub save_page {
 	$self->{save_lock} = 1; # Used to block autosave while "could not save" dialog
 	
 	my $page = $self->{page};
-	$self->{objects}{PageView}->save_page($page);
-	my $tree = $page->parse_tree;
+	my $tree = $self->{objects}{PageView}->get_parse_tree;
 	
-	eval {
+	eval { # FIXME maybe this logic belongs in PageView
 		my $old_status = $page->status;
 		if (@$tree > 2) { # content
-			$self->{repository}->save_page($page);
+			$page->set_parse_tree($tree);
 			$self->signal_emit('page_created', $page->name)
 				if $old_status eq 'new' or $old_status eq 'deleted';
 		}
 		else {
-			$self->{repository}->delete_page($page);
+			$page->delete;
 			$self->signal_emit('page_deleted', $page->name);
 		}
 	};
@@ -664,7 +882,7 @@ sub save_page {
 		return 0 unless defined $page;
 			# leave lock in position after returning
 		# FIXME check page exists
-		$self->{page} = $self->{repository}->load_page($page);
+		$self->{page} = $self->{repository}->open_page($page);
 		$self->save_page('FORCE'); # recurs
 	}
 
@@ -693,8 +911,10 @@ sub move_page {
 	}
 	else { $self->save_page }
 	
-	$to = $self->prompt_move_page_dialog($from)
-		unless defined $to;
+	unless (defined $to) {
+		$to = $self->prompt_move_page_dialog($from);
+		$to = $self->{repository}->resolve_page($to);
+	}
 	
 	return unless defined $to;
 
@@ -833,48 +1053,6 @@ sub new_button {
 	return $button;
 }
 
-=item C<add_button(TEXT, KEY, STOCK, CODE)>
-
-Add a button to the toolbar.
-
-=cut
-
-sub add_button {
-	my ($self, $text, $key, $stock, $code) = @_;
-	my $icon = Gtk2::Image->new_from_stock($stock, $self->{settings}{icon_size});
-	my $button = $self->{toolbar}->append_item(
-		#  text, tooltip_text, tooltip_private_text, icon, callback
-		$text, $text.($key?" ($key)":''), 'Foo Bar', $icon, $code);
-	if ($key) {
-		my $mod = [];
-		($mod, $key) = split ' ', $key, 2 if $key =~ / /;
-		if    ($mod eq 'Alt')  { $mod = 'mod1-mask'    }
-		elsif ($mod eq 'Ctrl') { $mod = 'control-mask' }
-		$key = $Gtk2::Gdk::Keysyms{$key} || ord($key);
-		$button->add_accelerator(
-			'clicked', $self->{accels}, $key, $mod, 'visible');
-	}
-	return $button;
-}
-
-=item C<add_key(KEY, CODE)>
-
-Add a key accelerator.
-
-=cut
-
-sub add_key {
-	my ($self, $key, $code) = @_;
-	
-	my $mod = [];
-	push @$mod, 'control-mask' if $key =~ s/^ctrl //i;
-	push @$mod, 'mod1-mask'    if $key =~ s/^alt //i;
-	push @$mod, 'shift-mask'   if $key =~ s/^shift //i;
-	$key = $Gtk2::Gdk::Keysyms{$key} || ord($key);
-
-	$self->{accels}->connect($key, $mod, 'visible', $code);
-}
-
 =item toggle_pane(BOOLEAN)
 
 Show or hide the side pane with the index tree.
@@ -889,13 +1067,12 @@ sub toggle_pane {
 	my $hpaned = $self->{hpaned};
 	if ($pane_visible) {
 		$hpaned->set_position($self->{settings}{pane_pos});
-		$tree_view->show;
-		$tree_view->load_index();
+		$tree_view->widget->show_all;
 		$tree_view->grab_focus;
 	}
 	else {
 		$self->{settings}{pane_pos} = $hpaned->get_position();
-		$tree_view->hide;
+		$tree_view->widget->hide_all;
 		$self->{objects}{PageView}->grab_focus;
 	}
 	$self->{settings}{pane_vis} = $pane_visible
@@ -910,13 +1087,15 @@ Set or cycle the type of the pathbar.
 
 sub toggle_path_bar {
 	my $self = shift;
-	my $type = $_[0] ? shift :
-		($self->{settings}{pathbar_type} eq 'trace')     ? 'namespace' :
-		($self->{settings}{pathbar_type} eq 'namespace') ? 'hidden'    : 'trace';
+	my $type = $self->{settings}{pathbar_type};
+	$type = shift || (
+		($type eq 'recent'   ) ? 'history'   :
+		($type eq 'history'  ) ? 'namespace' :
+		($type eq 'namespace') ? 'hidden'    : 'recent' );
 	$self->{settings}{pathbar_type} = $type;
 	
 	my $path_bar = $self->{objects}{PathBar};
-	if ($type eq 'trace' or $type eq 'namespace') {
+	if (grep {$type eq $_} qw/recent history namespace/) {
 		$path_bar->widget->show_all;
 		$path_bar->set_type($type);
 	}
@@ -985,17 +1164,21 @@ EOT
 	return $ok;
 }
 
-=item C<goto_page_dialog()>
+=item C<jump_page_dialog()>
 
 Prompts the user for a page name. This page is then opened.
 
 =cut
 
-sub goto_page_dialog {
+sub jump_page_dialog {
 	my $self = shift;
 	my $page = $self->{page}->namespace;
-	$page = $self->prompt_page_dialog($page, 'Go to', '_Go to Page:');
-	$self->link_clicked($page) if defined $page;
+	my $button = Gtk2::Button->new_from_stock('gtk-jump-to');
+	$page = $self->prompt_page_dialog(
+		$page, 'Jump to', '_Jump to Page:', undef, undef, $button);
+	return unless defined $page;
+	$page = $self->{repository}->resolve_page($page);
+	$self->load_page($page);
 }
 
 =item C<prompt_page_dialog(PAGE, TITLE, PROMPT, TEXT, ICON)>
@@ -1007,18 +1190,22 @@ The I<text> and I<icon> arguments are optional.
 =cut
 
 sub prompt_page_dialog {
-	my ($self, $page, $title, $prompt, $text, $icon) = @_;
+	my ($self, $page, $title, $prompt, $text, $icon, $button) = @_;
 
+	$page = '' if $page eq ':';
+	
 	my $dialog = Gtk2::Dialog->new(
 		$title, $self->{window},
 	       	[qw/modal destroy-with-parent no-separator/],
 		'gtk-cancel' => 'cancel',
-		'gtk-open'   => 'ok',
+		( defined($button) ? () : ('gtk-open'   => 'ok') )
 	);
 	$dialog->set_resizable(0);
 	$dialog->set_border_width(5);
 	$dialog->set_icon($self->{window}->get_icon);
-	$dialog->set_default_response('ok');
+	$dialog->add_action_widget($button, 'ok') if defined $button;
+	$dialog->set_default_response('ok') unless defined $button;
+		# the unless ... surpresses a warning ... how to put button as default ?
 
 	if (defined $text) {
 		my $text_hbox = Gtk2::HBox->new(0, 5);
@@ -1059,7 +1246,7 @@ sub prompt_page_dialog {
 
 =cut
 
-sub prompt_move_page_dialog {
+sub prompt_move_page_dialog { # TODO base this dialog on prompt_page_dialog()
 	my ($self, $page) = @_;
 	
 	my $title = "Move page";
@@ -1195,50 +1382,61 @@ it has a button to open the manual.
 
 =cut
 
-sub about_dialog {
+sub about_dialog { # TODO use Gtk2::AboutDialog for gtk > 2.6
 	my $self = shift;
 	
 	my $dialog = Gtk2::Dialog->new(
 		'About Zim', $self->{window},
-	       	[qw/modal destroy-with-parent/],
-		'gtk-help' => 'help',
-		'gtk-ok' => 'ok',
+	       	[qw/modal destroy-with-parent no-separator/],
+		'gtk-close' => 'close',
 	);
 	$dialog->set_resizable(0);
 	$dialog->set_border_width(5);
 	$dialog->set_icon($self->{window}->get_icon);
-	$dialog->set_default_response('ok');
+	my $button = $self->new_button('gtk-help', '_More');
+	$dialog->add_action_widget($button, 'help');
+	$dialog->set_default_response('close');
 	
 	$dialog->vbox->add(
 		Gtk2::Image->new_from_file($self->{settings}{icon_file}) );
 
 	my $text = $LONG_VERSION;
-	$text =~ s/^(.*)$/<b>$1<\/b>/m; # set first line bold
-	$text .= "\n\nThe current config file is: $self->{settings}{conf_file}\n";
-	$text .= "\nThe repository config is: $self->{repository}{conffile}\n"
-		if exists $self->{repository}{conffile};
+	$text =~ s/^(.*)$/<b>$1<\/b>/m;
 	my $label = Gtk2::Label->new();
 	$label->set_markup($text);
 	$label->set_justify('center');
 	$dialog->vbox->add($label);
 	
 	$dialog->show_all;
-	if ($dialog->run eq 'help') {
-		unless (fork) { # child process
-			no warnings;
-			@{$self->{settings}}{'width', 'height'} = $self->{window}->get_size;
-			$self->save_config(); # maybe we are read_only too
-			exec $0, '--doc';
-			# backup procedure to find perl
-			# when $0 is not executable for some reason
-			eval 'use Config';
-			exit 1 if $@;
-			my $perl = $Config{perl5} || 'perl';
-			exec $perl, $0, '--doc';
-			exit 1; # just to be sure
-		}
-	}
+	my $response = $dialog->run;
 	$dialog->destroy;
+
+	$self->show_help('zim:about') if $response eq 'help';
+}
+
+=item C<show_help(PAGE)>
+
+Spawns a new zim process showing the documentation. PAGE is optional.
+
+=cut
+
+sub show_help {
+	my ($self, $page) = @_;
+	
+	unless (fork) { # child process
+		no warnings;
+		@{$self->{settings}}{'width', 'height'} = $self->{window}->get_size;
+		$self->save_config(); # maybe we are read_only too
+		my @exec = ($0, '--doc', $page ? $page : ());
+		exec @exec;
+		# backup procedure to find perl
+		# when $0 is not executable for some reason
+		eval 'use Config';
+		exit 1 if $@;
+		my $perl = $Config{perl5} || 'perl';
+		exec @exec;
+		exit 1; # just to be sure
+	}
 }
 
 =item C<filechooser_dialog(FILE, TITLE)>
@@ -1248,22 +1446,22 @@ Ask the user for a filename. FILE is the suggested filename.
 =cut
 
 sub filechooser_dialog {
-	my ($self, $file, $title) = @_;
+	my ($self, $file, $dir, $title) = @_;
 	
 	my $dialog;
 	$title ||= 'Select File';
-	if (Gtk2->CHECK_VERSION(2, 4, 0) and $Gtk2::VERSION >= 1.040) {
-		$dialog = Gtk2::FileChooserDialog->new(
-			$title, $self->{window}, 'open',
-			'gtk-cancel' => 'cancel',
-			'gtk-ok'     => 'ok'
-		);
-	}
-	else { # old & ugly interface
-		$dialog = Gtk2::FileSelection->new($title);
-	}
+	# if (Gtk2->CHECK_VERSION(2, 4, 0) and $Gtk2::VERSION >= 1.040) {
+	$dialog = Gtk2::FileChooserDialog->new(
+		$title, $self->{window}, 'open',
+		'gtk-cancel' => 'cancel',
+		'gtk-ok'     => 'ok'
+	);
+	# }
+	#else { # old & ugly interface
+	#	$dialog = Gtk2::FileSelection->new($title);
+	#}
 	$dialog->set_icon($self->{window}->get_icon) if $self->{window};
-	$dialog->set_action('select-folder');
+	$dialog->set_action('select-folder') if $dir;
 	$dialog->set_filename(File::Spec->rel2abs($file)) if defined $file;
 	$dialog->signal_connect('response', sub {
 		$file = $_[1] eq 'ok' ? $dialog->get_filename : undef;
@@ -1337,9 +1535,9 @@ EOT
 =cut
 
 sub prompt_browser_dialog {
-	my $self = shift;
+	my ($self, $type) = @_;
 	
-	my $title = "Choose browser";
+	my $title = ($type eq 'file_browser') ? "Choose file browser" : "Choose browser";
 	my $dialog = Gtk2::Dialog->new(
 		$title, $self->{window},
 	       	[qw/modal destroy-with-parent no-separator/],
@@ -1351,15 +1549,15 @@ sub prompt_browser_dialog {
 	$dialog->set_icon($self->{window}->get_icon);
 	$dialog->set_default_response('ok');
 
-	my $label = Gtk2::Label->new(<< "EOT");
-Please enter a browser
-to open external links.
-EOT
+	my $label = Gtk2::Label->new(
+		($type eq 'file_browser')
+			? "Please enter a file browser"
+			: "Please enter a browser\nto open external links." );
 	$dialog->vbox->add($label);
 	
 	my $entry = Gtk2::Entry->new();
 	$entry->signal_connect(activate => sub { $dialog->response('ok') });
-	$entry->set_text("firefox '\%s'");
+	$entry->set_text(($type eq 'file_browser') ? "rox '\%s'" : "firefox '\%s'");
 	$dialog->vbox->add($entry);
 
 	$dialog->show_all;
