@@ -3,13 +3,12 @@ package Zim::Components::PageView;
 use strict;
 use vars '$AUTOLOAD';
 use Gtk2;
-#use Gtk2::Spell;
 use Gtk2::Pango;               # pango constants
 use Gtk2::Ex::HyperTextView;   # custom widget
 use Gtk2::Ex::HyperTextBuffer; #    "     "
 use POSIX qw(strftime);
 
-our $VERSION = 0.07;
+our $VERSION = '0.10';
 
 # TODO move more logic into Gtk2::Ex::HyperTextBuffer !
 
@@ -32,8 +31,112 @@ Undefined methods are AUTOLOADED to the Gtk2::Ex::HyperTextView object.
 
 =cut
 
-my ($k_tab, $k_return, $k_kp_enter, $k_backspace, $k_escape, $k_multiply) =
-	@Gtk2::Gdk::Keysyms{qw/Tab Return KP_Enter BackSpace Escape KP_Multiply/};
+our $LINK_ICON = Gtk2->CHECK_VERSION(2, 6, 0) ? 'gtk-connect' : 'gtk-convert';
+	# gtk-connect stock item was introduced in 2.6.0
+
+my @ui_actions = (
+	# name,          stock id,         label,            accelerator,       tooltip
+	[ 'Head1',       undef,            'Head _1',        '<ctrl>1',         'Heading 1',     \&on_tag         ],
+	[ 'Head2',       undef,            'Head _2',        '<ctrl>2',         'Heading 2',     \&on_tag         ],
+	[ 'Head3',       undef,            'Head _3',        '<ctrl>3',         'Heading 3',     \&on_tag         ],
+	[ 'Head4',       undef,            'Head _4',        '<ctrl>4',         'Heading 4',     \&on_tag         ],
+	[ 'Head5',       undef,            'Head _5',        '<ctrl>5',         'Heading 5',     \&on_tag         ],
+	[ 'Normal',      undef,            '_Normal',        '<ctrl>6',         'Normal',        \&on_tag         ],
+	[ 'Bold',        'gtk-bold',       '_Bold',          '<ctrl>B',         'Bold',          \&on_tag         ],
+	[ 'Italic',      'gtk-italic',     '_Italic',        '<ctrl>I',         'Italic',        \&on_tag         ],
+	[ 'Underline',   'gtk-underline',  '_Underline',     '<ctrl>U',         'Underline',     \&on_tag         ],
+	[ 'Verbatim',    undef,            '_Verbatim',      '<ctrl>T',         'Verbatim',      \&on_tag         ],
+	[ 'Undo',        'gtk-undo',       '_Undo',          '<ctrl>Z',         'Undo',          \&on_Undo        ],
+	[ 'Redo',        'gtk-redo',       '_Redo',          '<ctrl><shift>Z',  'Redo',          \&on_Redo        ],
+	[ 'Cut',         'gtk-cut',        'Cu_t',           '<ctrl>X',         'Cut',           \&on_clipboard   ],
+	[ 'Copy',        'gtk-copy',       '_Copy',          '<ctrl>C',         'Copy',          \&on_clipboard   ],
+	[ 'Paste',       'gtk-paste',      '_Paste',         '<ctrl>V',         'Paste',         \&on_clipboard   ],
+	[ 'Delete',      'gtk-delete',     'Delete',         'Delete',          'Delete',        \&on_Delete      ],
+	[ 'EditLink',    'gtk-properties', '_Link',          '<ctrl>E',         'Edit link',     \&on_EditLink    ],
+	[ 'InsertDate',  undef,            '_Date and Time', '<ctrl>D',         'Insert date',   \&on_InsertDate  ],
+	[ 'InsertLink',  $LINK_ICON,       '_Link...',       undef,             'Insert link',   \&on_EditLink    ],
+	[ 'InsertImage', undef,            '_Image...',      undef,             'Insert image',  \&on_InsertImage ],
+);
+
+my @ui_actions_ro = (
+	[ 'Link',        $LINK_ICON,       '_Link',          '<ctrl>L',         'Link',          \&on_Link        ],
+	[ 'Find',        'gtk-find',       '_Find',          '<ctrl>F',         'Find',          \&on_Find        ],
+	[ 'FindNext',    undef,            'Find Ne_xt',     '<ctrl>G',         'Find next',     \&on_FindNext    ],
+	[ 'FindPrev',    undef,            'Find Pre_vious', '<ctrl><shift>G',  'Find previous', \&on_FindPrev    ],
+);
+
+my $ui_layout = q{<ui>
+	<menubar name='MenuBar'>
+		<menu action='EditMenu'>
+			<menuitem action='Undo'/>
+			<menuitem action='Redo'/>
+			<separator/>
+			<menuitem action='Cut'/>
+			<menuitem action='Copy'/>
+			<menuitem action='Paste'/>
+			<menuitem action='Delete'/>
+			<separator/>
+			<menuitem action='EditLink'/>
+			<separator/>
+		</menu>
+		<menu action='InsertMenu'>
+			<menuitem action='InsertDate'/>
+			<separator/>
+			<menuitem action='InsertImage'/>
+			<menuitem action='InsertLink'/>
+		</menu>
+		<menu action='FormatMenu'>
+			<menuitem action='Head1'/>
+			<menuitem action='Head2'/>
+			<menuitem action='Head3'/>
+			<menuitem action='Head4'/>
+			<menuitem action='Head5'/>
+			<menuitem action='Normal'/>
+			<separator/>
+			<menuitem action='Bold'/>
+			<menuitem action='Italic'/>
+			<menuitem action='Underline'/>
+			<menuitem action='Verbatim'/>
+			<separator/>
+			<menuitem action='Link'/>
+		</menu>
+	</menubar>
+	<toolbar name='ToolBar'>
+		<placeholder name='Format'>
+			<toolitem action='Link'/>
+			<toolitem action='EditLink'/>
+			<separator/>
+			<toolitem action='Bold'/>
+			<toolitem action='Italic'/>
+			<toolitem action='Underline'/>
+		</placeholder>
+		<separator/>
+	</toolbar>
+</ui>};
+
+my $ui_layout_ro = q{<ui>
+	<menubar name='MenuBar'>
+		<menu action='EditMenu'>
+			<menuitem action='Find'/>
+			<menuitem action='FindNext'/>
+			<menuitem action='FindPrev'/>
+		</menu>
+	</menubar>
+	<accelerator action='Link'/>
+</ui>};
+
+=item C<new(app => PARENT)>
+
+Simple constructor.
+
+=item C<init()>
+
+Method called by the constructor.
+
+=cut
+
+my ($k_tab, $k_return, $k_kp_enter, $k_backspace, $k_escape, $k_multiply, $k_home) =
+	@Gtk2::Gdk::Keysyms{qw/Tab Return KP_Enter BackSpace Escape KP_Multiply Home/};
 
 our %UNDO_STEPS = (
 	delete     => 'insert',
@@ -42,19 +145,6 @@ our %UNDO_STEPS = (
 	remove_tag => 'apply_tag'
 );
 
-our %TAG_ALIASES = (
-	B         => 'bold',
-	C         => 'verbatim',
-	I         => 'italic',
-	L         => 'link',
-	U         => 'underline',
-	bold      => 'B',
-	verbatim  => 'C',
-	italic    => 'I',
-	link      => 'L',
-	underline => 'U',
-);
-
 $Gtk2::Ex::HyperTextBuffer::TAGS{link} = [foreground => 'blue'];
 $Gtk2::Ex::HyperTextBuffer::TAGS{underline} = [background => 'yellow'];
 
@@ -69,14 +159,14 @@ sub init { # called by new()
 	my $self = shift;
 	$self->{overwrite_mode} = 0;
 
-	my $vbox = Gtk2::VBox->new(0, 3);
+	my $vbox = Gtk2::VBox->new(0, 0);
 	$self->{vbox} = $vbox;
 	
-	my $scroll_window = Gtk2::ScrolledWindow->new();
-	$scroll_window->set_policy('automatic', 'automatic');
-	$scroll_window->set_shadow_type('in');
-	$vbox->add($scroll_window);
-	$self->{scroll_window} = $scroll_window;
+	my $scrolled_window = Gtk2::ScrolledWindow->new();
+	$scrolled_window->set_policy('automatic', 'automatic');
+	$scrolled_window->set_shadow_type('in');
+	$vbox->add($scrolled_window);
+	$self->{scrolled_window} = $scrolled_window;
 	
 	# init TextView
 	my $htext = Gtk2::Ex::HyperTextView->new();
@@ -96,16 +186,20 @@ sub init { # called by new()
 	$htext->signal_connect(toggle_overwrite => \&on_toggle_overwrite, $self);
 	$htext->signal_connect(populate_popup => \&on_populate_popup);
 	$htext->signal_connect(key_press_event => \&on_key_press_event, $self);
-	$scroll_window->add($htext);
+	my $reset_em = sub {
+		$self->{buffer}->reset_edit_mode if $self->{buffer};
+		return 0;
+	};
+	$htext->signal_connect(move_cursor => $reset_em);
+	$htext->signal_connect(button_press_event => $reset_em);
+	$scrolled_window->add($htext);
 	$self->{htext} = $htext;
 
-	#my $spell = Gtk2::Spell->new_attach($htext);
-	#$self->{spell} = $spell;
-
 	# init search box
 	my $hbox = Gtk2::HBox->new(0, 5);
-	$vbox->pack_start($hbox, 0, 1, 0);
+	$hbox->set_no_show_all(1);
 	$hbox->signal_connect(key_press_event => \&on_key_press_event_hbox);
+	$vbox->pack_start($hbox, 0, 1, 3);
 	$self->{hbox} = $hbox;
 
 	$hbox->pack_start( Gtk2::Label->new(' Find: '), 0,1,0);
@@ -116,92 +210,33 @@ sub init { # called by new()
 	$hbox->pack_start($entry, 0, 1, 0);
 	$self->{entry} = $entry;
 
-	my $prev_button = $self->{app}->new_button('gtk-go-back', 'Previous');
+	my $prev_button = $self->{app}->new_button('gtk-go-back', '_Previous');
 	$prev_button->signal_connect(clicked =>
 		sub { $self->find($entry->get_text, -1) } );
 	$hbox->pack_start($prev_button, 0, 1, 0);
 	
-	my $next_button = $self->{app}->new_button('gtk-go-forward', 'Next');
+	my $next_button = $self->{app}->new_button('gtk-go-forward', '_Next');
 	$next_button->signal_connect(clicked =>
 		sub { $self->find($entry->get_text, 1) } );
 	$hbox->pack_start($next_button, 0, 1, 0);
 
 	# add toolbar buttons and key bindings
+	my $accels = $self->{app}{ui}->get_accel_group;
 	my $read_only = $self->{app}{settings}{read_only};
-	unless ($read_only) { # Style menu  # FIXME better menu layout
-		my $style_menu = Gtk2::Menu->new;
-		$style_menu->append( Gtk2::TearoffMenuItem->new() );
-		for my $tag (
-			[ 'Head 1',   'Ctrl 1' => 'head1'     ],
-			[ 'Head 2',   'Ctrl 2' => 'head2'     ],
-			[ 'Head 3',   'Ctrl 3' => 'head3'     ],
-			[ 'Head 4',   'Ctrl 4' => 'head4'     ],
-			[ 'Head 5',   'Ctrl 5' => 'head5'     ],
-			[ 'Normal',   'Ctrl 6' => 'Normal'    ],
-			[ 'Bold',     'Ctrl B' => 'bold'      ],
-			[ 'Italic',   'Ctrl I' => 'italic'    ],
-			[ 'Mark',     'Ctrl U' => 'underline' ],
-			[ 'Verbatim', 'Ctrl T' => 'verbatim'  ],
-		) {
-			my $code = sub { $self->apply_tag($$tag[2]) };
-		
-			my $item_hbox = Gtk2::HBox->new(0,5);
-			# FIXME do something with alignments etc.
-			$item_hbox->pack_start(Gtk2::Label->new($$tag[0]), 1,1,0);
-			$item_hbox->pack_start(Gtk2::Label->new("($$tag[1])"), 0,1,0);
-			my $item = Gtk2::MenuItem->new();
-			$item->add($item_hbox);
-			$item->signal_connect(activate => $code);
-			$style_menu->append($item);
-
-			$self->{app}->add_key($$tag[1] => $code);
-		}
-		my $item = Gtk2::MenuItem->new('Bullet list');
-		$item->signal_connect(activate => sub {$self->toggle_bullets});
-		$style_menu->append($item);
-		
-		$style_menu->show_all;
-
-		my $style_button = $self->{app}->add_button(
-			'Styles', 'Alt T', 'gtk-select-font', sub {
-			$style_menu->popup(undef, undef, undef, undef, 1, 0);
-		} );
-		$style_button->signal_connect( button_press_event => sub {
-			my (undef, $event) = @_;
-			return 0 unless $event->button == 1;
-			
-			my ($x, $y) = $style_button->get_parent_window->get_position;
-			my $allocation = $style_button->allocation;
-			$x += $allocation->x;
-			$y += $allocation->y + $allocation->height;
-
-			$style_menu->popup(
-				undef, undef, sub {$x, $y}, undef, $event->button, $event->time);
-			return 1;
-		});
-		$style_menu->attach_to_widget($style_button, 0);
+	my $actions = Gtk2::ActionGroup->new("PageView");
+	$self->{app}{ui}->insert_action_group($actions, 0);
+	$actions->add_actions(\@ui_actions_ro, $self);
+	unless ($read_only) {
+		$actions->add_actions(\@ui_actions, $self);
+		$self->{app}{ui}->add_ui_from_string($ui_layout);
+		$accels->connect( # ^Y (identical with the shift-^Z defined above)
+			ord('Y'), ['control-mask'], ['visible'], sub {$self->redo} );
 	}
+	$self->{app}{ui}->add_ui_from_string($ui_layout_ro);
+	$accels->connect( # alt-/ (identical with the ^F defined above)
+		ord('/'), ['mod1-mask'], ['visible'], sub {$self->show_find} );
 
-	$self->{app}->add_button(
-		'Link', 'Ctrl L', $Zim::LINK_ICON, sub {$self->apply_link});
-	$self->{app}->add_button(
-		'Edit Link', 'Ctrl E', 'gtk-properties',
-		sub {$self->edit_link_dialog} ) unless $read_only;
-	#$self->{app}->add_button(
-	#	'Insert image', undef, 'gtk-select-color',
-	#	sub {$self->{app}->filechooser_dialog} ) unless $read_only;
-
-	$self->{app}->add_key('Alt /', sub {$self->show_search} );
-
-	unless ($read_only) { # Some more keybindings
-		$self->{app}->add_key('Ctrl Z', sub {$self->undo});
-		$self->{app}->add_key('Ctrl Y', sub {$self->redo});
-		my $date_string = $self->{app}{settings}{date_string};
-		$self->{app}->add_key('Ctrl D', sub {
-			$self->get_buffer->insert_at_cursor(
-				strftime($date_string, localtime) )
-		} );
-	}
+	$self->{htext}->set_cursor_visible( $read_only ? 0 : 1 );
 }
 
 sub AUTOLOAD {
@@ -210,7 +245,43 @@ sub AUTOLOAD {
 	return if $AUTOLOAD eq 'DESTROY';
 	return $self->{htext}->$AUTOLOAD(@_);
 }
-	
+
+sub on_tag {
+	my ($action, $self) = @_;
+	my $tag = lc $action->get_name;
+	$self->apply_tag($tag);
+}
+
+sub on_Link { pop->apply_link }
+
+sub on_Undo { pop->undo }
+
+sub on_Redo { pop->redo }
+
+sub on_clipboard {
+	my ($action, $self) = @_;
+	my $signal = lc($action->get_name).'_clipboard';
+	$self->{htext}->signal_emit($signal);
+}
+
+sub on_Delete { pop->{htext}->signal_emit('delete_from_cursor', 'chars', 1) }
+
+sub on_EditLink { pop->edit_link_dialog }
+
+sub on_Find { pop->show_find }
+
+sub on_FindNext { pop->find(undef, 1) }
+
+sub on_FindPrev { pop->find(undef, -1) }
+
+sub on_InsertDate {
+	my $self = pop;
+	$self->get_buffer->insert_at_cursor(
+		strftime( $self->{app}{settings}{date_string}, localtime) );
+}
+
+sub on_InsertImage { pop->insert_image }
+
 =item C<widget()>
 
 Returns the root widget. This should be used to add the object to a container widget.
@@ -229,7 +300,7 @@ sub on_key_press_event_hbox {
 
 sub on_changed_entry {
 	my ($self, $entry) = @_;
-	$self->find($entry->get_text);
+	$self->find($entry->get_text, 0);
 }
 
 sub on_activate_entry {
@@ -254,16 +325,13 @@ sub find {
 	my $iter = $buffer->get_iter_at_mark( $buffer->get_insert );
 	$iter->forward_char if $direction == 1;
 	
-	my ($start, $end);
-	unless ($direction == -1) { # forward
-		($start, $end) = $iter->forward_search($string, []);
-		unless (defined $start) { # wrap around
-			$iter = $buffer->get_start_iter;
-			($start, $end) = $iter->forward_search($string, []);
-			return unless defined $start;
-		}
+	unless (defined $string) {
+		# TODO also use selection here - think ^G while entry is hidden - also set entry when selection
+		$string = $self->{entry}->get_text;
 	}
-	else { # backward
+	
+	my ($start, $end);
+	if ($direction == -1) { # backward
 		($start, $end) = $iter->backward_search($string, 'visible-only');
 		unless (defined $start) { # wrap around
 			$iter = $buffer->get_end_iter;
@@ -271,30 +339,57 @@ sub find {
 			return unless defined $start;
 		}
 	}
+	else { # forward (direction 1 or 0)
+		($start, $end) = $iter->forward_search($string, []);
+		unless (defined $start) { # wrap around
+			$iter = $buffer->get_start_iter;
+			($start, $end) = $iter->forward_search($string, []);
+			return unless defined $start;
+		}
+	}
 	
 	#print "found $string at offset ".$iter->get_offset."\n";
-	$buffer->place_cursor($start);
-	$buffer->move_mark($buffer->get_selection_bound, $end);
+	$self->set_selection($start, $end);
 	$self->{htext}->scroll_mark_onscreen( $buffer->get_insert );
 }
 
-=item C<show_search()>
+=item C<set_selection(START, END)>
+
+Select a region of the buffer.
 
 =cut
 
-sub show_search {
+sub set_selection {
+	my ($self, $start, $end) = @_;
+	my $buffer = $self->{htext}->get_buffer;
+	$buffer->place_cursor($start);
+	$buffer->move_mark( $buffer->get_selection_bound, $end );
+#	$buffer->select_range($start, $end); # Since 2.4 FIXME
+}
+
+=item C<show_find()>
+
+Show the search bar.
+
+=cut
+
+sub show_find {
 	my $self = shift;
+	$self->{hbox}->set_no_show_all(0);
 	$self->{hbox}->show_all;
 	$self->{entry}->grab_focus;
 }
 
-=item C<hide_search()>
+=item C<hide_find()>
+
+Hide the search bar.
 
 =cut
 
-sub hide_search {
+sub hide_find {
 	my $self = shift;
 	$self->{hbox}->hide_all;
+	$self->{hbox}->set_no_show_all(1);
 	$self->{htext}->grab_focus;
 }
 
@@ -304,22 +399,35 @@ sub on_toggle_overwrite {
 	$self->{app}->update_status();
 }
 
-sub on_populate_popup { # add items to context menu
+sub on_populate_popup { # FIXME add this code to HyperTextView
+       	# add items to context menu
 	my ($htext, $menu) = @_;
-	my $buffer = $htext->get_buffer;
-	my $link = $htext->get_link_at_pointer();
-	return unless $link;
-	$link = $$link[1];
+	my $iter = $htext->get_iter_at_pointer;
+	return unless $iter;
+	
+	my $string;
+	my $link = $htext->get_link_at_iter($iter);
+	if (defined $link) {
+		$link = $$link[1] if ref($link) eq 'ARRAY';
+		$string = ($link =~ s/^mailto:\/*//)
+			? 'Copy _Email Address'
+			: 'Copy _Link' ;
+	}
+	else {
+		my $pixbuf = $iter->get_pixbuf;
+		return unless $pixbuf and defined $pixbuf->{image_src};
+		$link = $pixbuf->{image_src};
+		$string = 'Copy Image _Location';
+	}
 	
 	my $seperator = Gtk2::MenuItem->new();
 	$seperator->show;
 	$menu->prepend($seperator);
 	
-	my $item = Gtk2::MenuItem->new(
-		($link =~ s/^mailto:\/*//) ? 'Copy _Email Address' : 'Copy _Link' );
+	my $item = Gtk2::MenuItem->new($string);
 	$item->signal_connect(activate => sub {
 		my $clipboard = Gtk2::Clipboard->get(
-			Gtk2::Gdk::Atom->new('PRIMARY', 1) );
+			Gtk2::Gdk::Atom->new('PRIMARY') );
 		$clipboard->set_text($link);
 	} );
 	$item->show;
@@ -333,7 +441,7 @@ sub on_key_press_event { # some extra keybindings
 
 	if ($self->{app}{settings}{read_only}) {
 		return 0 unless $val == ord('/');
-		$self->show_search;
+		$self->show_find;
 		return 1;
 	}
 	
@@ -345,12 +453,15 @@ sub on_key_press_event { # some extra keybindings
 		$htext->scroll_mark_onscreen( $buffer->get_insert );
 		return 1;
 	}
-	elsif ($val == $k_backspace) { # BackSpace
+	elsif (
+		$self->{app}{settings}{backsp_unindent} and $val == $k_backspace or
+		$val == $k_tab and $event->state eq 'shift-mask'
+	) { # BackSpace or Shift-Tab
 		my $buffer = $htext->get_buffer;
 		my ($start, $end) = $buffer->get_selection_bounds;
 		if ($end and $end != $start) {
-			$self->selection_backspace($start, $end);
-			return 1;
+			my $cont = $self->selection_backspace($start, $end);
+			return $val == $k_tab ? 1 : $cont;
 		}
 		my $iter = $buffer->get_iter_at_mark($buffer->get_insert());
 		if ($self->parse_backspace($iter)) {
@@ -381,6 +492,29 @@ sub on_key_press_event { # some extra keybindings
 		$self->toggle_bullets($start, $end);
 		return 1;
 	}
+	elsif ($val == $k_home and not $event->state >= 'control-mask') { # Home toggle
+		my $buffer = $htext->get_buffer;
+		my $insert = $buffer->get_iter_at_mark($buffer->get_insert());
+		my $start  = $insert->copy;
+		$htext->backward_display_line_start($start)
+			unless $htext->starts_display_line($start);
+		my $begin  = $start->copy;
+		my $indent = '';
+		while ($indent =~ /^\s*([^\s\w]\s*)?$/) {
+			last if $begin->ends_line or ! $begin->forward_char;
+			$indent = $start->get_text($begin);
+		}
+		$indent =~ /^(\s*([^\s\w]\s+)?)/;
+		my $back = length($indent) - length($1);
+		$begin->backward_chars($back) if $back > 0;
+		$insert = ($begin->ends_line || $insert->equal($begin)) ? $start : $begin;
+		if ($event->state >= 'shift-mask') {
+			$buffer->move_mark_by_name('insert', $insert);
+			# leaving the "selection_bound" mark behind
+		}
+		else { $buffer->place_cursor($insert) }
+		return 1;
+	}
 		
 	#else { printf "key %x pressed\n", $val } # perldoc -m Gtk2::Gdk::Keysyms
 
@@ -395,9 +529,12 @@ Returns a number of properties that need to be saved in the history.
 
 sub get_state {
 	my $self = shift;
-	my $buffer = $self->{buffer};
+	my $buffer = $self->{buffer} || return;
 	my $cursor = $buffer->get_iter_at_mark($buffer->get_insert)->get_offset;
+	#my $vscroll = $self->{scrolled_window}->get_vadjustment->get_value;
+	#my $hscroll = $self->{scrolled_window}->get_hadjustment->get_value;
 	return cursor => $cursor, undo => $self->{undo}, redo => $self->{redo};
+		#, vscroll => $vscroll, hscroll => $hscroll;
 }
 
 =item C<set_state(property => value, ..)>
@@ -413,10 +550,13 @@ sub set_state {
 	if (defined $rec{cursor}) {
 		$self->{buffer}->place_cursor(
 			$self->{buffer}->get_iter_at_offset($rec{cursor}) );
-
+	
 		$self->{htext}->scroll_mark_onscreen( $self->{buffer}->get_insert );
 	}
 	
+	#$self->{scrolled_window}->get_vadjustment->set_value($rec{vscroll});
+	#$self->{scrolled_window}->get_hadjustment->set_value($rec{hscroll});
+	
 	$self->{undo} = $rec{undo} || [];
 	$self->{redo} = $rec{redo} || [];
 }
@@ -442,14 +582,20 @@ Load a new page object into the buffer.
 
 sub load_page {
 	my ($self, $page) = @_;
+	my $use_spell = defined $self->{app}{objects}{Spell}; # FIXME ugly internals
 	
-	# create a new TextBuffer
+	# clear the old buffer
+	$self->{undo_lock} = 1;
+	$self->{buffer}->clear if $self->{buffer};
+	$self->{app}->Spell->detach($self->{htext}) if $use_spell;
+	$self->{_prev_buffer} = $self->{buffer}; # FIXME hack to prevent segfaults
+	
+	# create a new HyperTextBuffer
 	my $buffer = Gtk2::Ex::HyperTextBuffer->new();
-	$buffer->tag_aliases(\%TAG_ALIASES);
+	$buffer->create_default_tags;
 	$self->{buffer} = $buffer;
 	$self->{htext}->set_buffer($buffer);
-	#$self->load_parsetree($page);
-	$buffer->set_parse_tree( $page->parse_tree );
+	$buffer->set_parse_tree( $page->get_parse_tree );
 	unless ($self->{app}{settings}{read_only}) {
 		# connect signals _after_ load_parsetree()
 		$buffer->signal_connect(delete_range => \&on_delete_range, $self);
@@ -459,6 +605,8 @@ sub load_page {
 		$buffer->signal_connect(apply_tag => \&on_apply_tag, $self);
 		$buffer->signal_connect(remove_tag => \&on_remove_tag, $self);
 	}
+	$self->{app}->Spell->attach($self->{htext}) if $use_spell;
+	$self->{undo_lock} = 0;
 	$buffer->set_modified(0);
 
 	$buffer->place_cursor(
@@ -487,17 +635,13 @@ sub modified {
 	$_[0]->{buffer}->get_modified;
 }
 
-=item C<save_page(PAGE)>
+=item C<get_parse_tree(PAGE)>
 
-Put the content of the buffer back in a page object.
-Returns true if the buffer is not empty.
+Returns the parse tree for the current buffer contents.
 
 =cut
 
-sub save_page {
-	my ($self, $page) = @_;	
-	$page->parse_tree( $self->{buffer}->get_parse_tree );
-}
+sub get_parse_tree { $_[0]->{buffer}->get_parse_tree }
 
 =item C<parse_backspace(ITER)>
 
@@ -571,11 +715,14 @@ sub parse_word {
 	return 0 unless $char eq ' ' or $char eq "\t";
 	return 0 if $self->_is_verbatim;
 	my $buffer = $self->{buffer};
-	my $lf = $buffer->get_iter_at_line( $iter->get_line );
+	my $lf = $iter->copy;
+	$self->{htext}->backward_display_line_start($lf)
+		unless $self->{htext}->starts_display_line($lf);
 	my $line = $buffer->get_text($lf, $iter, 0) . $char;
 	#print ">>$line<<\n";
 	if ($line =~ /^(\s*)[\*\x{2022}](\s+)$/) { # bullet
 		# FIXME \s* => \t
+		return unless $lf->starts_line;
 		my ($pre, $post) = ($1, $2);
 		$pre .= $1 if $post =~ s/(\t)+$//; # switch tabs
 		$line = $pre."\x{2022}".$post;
@@ -605,6 +752,20 @@ sub _is_verbatim {
 	return 0;
 }
 
+=item C<insert_image(IMAGE)>
+
+Inserts an image into the buffer. Without argument prompts the user for a file.
+
+=cut
+
+sub insert_image {
+	my ($self, $file) = @_;
+	$file = $self->{app}->filechooser_dialog unless $file;
+	return unless length $file;
+	# TODO check relativeness of filename etc.
+	$self->{buffer}->insert_image_from_file($file);
+}
+
 =item C<apply_tag(TAG)>
 
 Applies the tag with the name TAG to any selected text.
@@ -624,7 +785,7 @@ sub apply_tag {
 	my $buffer = $self->{buffer};
 	$buffer->remove_all_tags($start, $end);
 	$buffer->apply_tag_by_name($tag, $start, $end)
-		unless $tag eq 'Normal';
+		unless $tag eq 'normal';
 	$buffer->set_modified(1);
 
 	if ($tag =~ /^head/) { # give headers their own line
@@ -635,13 +796,18 @@ sub apply_tag {
 	}
 }
 
+=item C<get_selection(TAG)>
+
+=cut
+
 sub get_selection { # selects current word if no selections
 	my ($self, $tag) = @_;
 	my $buffer = $self->{buffer};
 	my ($start, $end) = $buffer->get_selection_bounds;
 #	print "selection: $start, $end\n";
 	if (!$end or $start == $end) {
-		# TODO autoselect word ( or line if tag =~ /^head/ );
+		# TODO autoselect word if defined tag
+		# ( or line if tag =~ /^head/ );
 		return undef;
 	}
 	return ($start, $end);
@@ -664,7 +830,7 @@ followed immediatly, but no actual link is made
 
 sub apply_link {
 	my ($self, $link) = @_;
-	my ($start, $end) = $self->get_selection('L');
+	my ($start, $end) = $self->get_selection('link');
 	
 	unless (defined $start) {
 		$self->{app}{settings}{read_only}
@@ -712,6 +878,10 @@ sub toggle_bullets {
 	$self->replace_selection($text, $start, $end);
 }
 
+=item C<replace_selection(TEXT, START, END)>
+
+=cut
+
 sub replace_selection {
 	my ($self, $text, $start, $end) = @_;
 	($start, $end) = $self->get_selection unless defined $start;
@@ -724,8 +894,7 @@ sub replace_selection {
 	$buffer->insert($start, $text);
 
 	($start, $end) = map $buffer->get_iter_at_offset($_), ($_start, $_end);
-	$buffer->place_cursor($start);
-	$buffer->move_mark($buffer->get_selection_bound, $end);
+	$self->set_selection($start, $end);
 
 	return $start, $end;
 }
@@ -761,18 +930,17 @@ sub selection_backspace {
 	my ($self, $start, $end) = @_;
 	($start, $end) = $self->get_selection unless defined $start;
 	return unless defined $start;
-	
+
 	my $text = $self->{buffer}->get_text($start, $end, 1);
 	my $verbatim = $self->_is_verbatim;
 	if ($text =~ s/^\t//mg) {
 		($start, $end) = $self->replace_selection($text, $start, $end);
 		$self->{buffer}->apply_tag_by_name('Verbatim', $start, $end)
 			if $verbatim;
+		return 1;
 	}
-	elsif ($verbatim) {
-		$self->{buffer}->remove_all_tags($start, $end);
-	}	
-	else { $self->replace_selection('', $start, $end) }
+
+	return 0;
 }
 
 sub on_insert_text { # buffer, iter, string, length, self
@@ -803,6 +971,10 @@ sub _on_change_tag {
 	}
 }
 
+=item C<add_undo(ACTION, OFFSET, DATA)>
+
+=cut
+
 sub add_undo {
 	my $self = shift;
 	return if $self->{undo_lock}; # prohibit unwanted recursion
