@@ -2,16 +2,12 @@ package Zim::Formats::Html;
 
 use strict;
 
-our $VERSION = 0.02;
+our $VERSION = '0.10';
 
 =head1 NAME
 
 Zim::Formats::Html - Html dumper for zim
 
-=head1 SYNOPSIS
-
-FIXME simple code example
-
 =head1 DESCRIPTION
 
 This is a dumper that can be used to export zim wiki pages to html.
@@ -20,40 +16,41 @@ This is a dumper that can be used to export zim wiki pages to html.
 
 =over 4
 
-=item C<save_tree(TREE, PAGE)>
+=item C<save_tree(IO, TREE, PAGE)>
 
 Converts a parse tree into plain text.
 
 =cut
 
-# WARNING: the api for exporting is not yet stable
-#          do not use this as a reference implementation .. yet
-
 # TODO avoid reading the template for ech page, cache it somewhere
 
-sub _extension { 'html' }
-
 sub save_tree {
-	my ($class, $tree, $page) = @_;
+	my ($class, $io, $tree, $page) = @_;
 
-	my $text = _dump_node($tree, $page);
-	
 	my $template = $page->{repository}{_template};
-	$template = _read_file($template);
-	$template =~ s/\%\%CONTENT\%\%/$text/;
-	return $template;
+	my ($head, $foot) = split /\%\%CONTENT\%\%/, _read_file($template), 2;
+	
+	my $old_fh = select $io;
+	eval {
+		print $head;
+		_dump_node($tree, $page);
+		print $foot;
+	};
+	select $old_fh;
+	die $@ if $@;
 }
 
 sub _dump_node {
 	my ($node, $page) = @_;
 	my ($name, $opt) = splice @$node, 0, 2;
 	
-	my ($text, @tags);
+	my @tags;
 	if ($name =~ /^head(\d+)/) { @tags = ("<h$1>", "</h$1>\n") }
 	elsif ($name eq 'Para') {
 		@tags = ("<p>\n", "</p>\n");
 		if (! ref $$node[0] and $$node[0] =~ /^\s*\*\s+/m) {
-			$text = parse_list($page, @$node);
+			# ugly hack ... should not be necessary
+			parse_list($page, @$node);
 			@$node = ();
 		}
 	}
@@ -74,51 +71,51 @@ sub _dump_node {
 			$href =~ s#:+#/#g;
 			$href .= '.html';
 		}
-		print "  link: $$opt{to} => $href\n";
+		print STDERR "  link: $$opt{to} => $href\n";
 		@tags = ("<a href='$href'>", "</a>");
 	}	
 
+	print $tags[0];
 	for (@$node) {
-		if (ref $_) { $text .= _dump_node($_, $page) } # recurse
+		if (ref $_) { _dump_node($_, $page) } # recurse
 		else {
 			$_ =~ s/([^-\n\t !\#\$\%\(\)\*\+,\.\~\/\:\;=\?\@\[\\\]\^_\`\{\|\}abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789])/'&#'.(ord($1)).';'/eg; # RegEx copy pasted from Pod::Simple::Html
-			$text .= $_;
+			print $_;
 		}
 	}
-	return $tags[0].$text.$tags[1];
+	print $tags[1];
 }
 
 sub parse_list { # FIXME totally wrong level for this kind of parsing :(
 	my ($page, @nodes) = @_;
 	
-	my ($text, $level, $close);
+	print "<ul>\n";
+	my ($level, $close);
 	for (@nodes) {
-		if (ref $_) { $text .= _dump_node($_, $page) }
+		if (ref $_) { _dump_node($_, $page) }
 		else {
 			$_ =~ s/([^-\n\t !\#\$\%\(\)\*\+,\.\~\/\:\;=\?\@\[\\\]\^_\`\{\|\}abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789])/'&#'.(ord($1)).';'/eg; # RegEx copy pasted from Pod::Simple::Html
 			my @lines = split /[\n\r]+/, $_;
 			for (@lines) {
 				if ($_ =~ s/^(\s*)\*\s+//) {
-					$text .= $close;
+					print $close;
 					my $lvl = length $1;
 					if ($lvl > $level) {
-						$text .= "<ul>\n" for 1 .. $lvl - $level;
+						print "<ul>\n" for 1 .. $lvl - $level;
 					}
 					elsif ($lvl < $level) {
-						$text .= "</ul>\n" for 1 .. $level - $lvl;
+						print "</ul>\n" for 1 .. $level - $lvl;
 					}
 					$level = $lvl;
-					$text  .= "<li>".$_;
+					print "<li>".$_;
 					$close = "</li>\n";
 				}
-				else { $text .= $_ }
+				else { print $_ }
 			}
 		}
 	}
-	$text .= $close;
-	$text .= "</ul>\n" for 1 .. $level;
-	
-	return "<ul>\n$text</ul>\n";
+	print $close;
+	print "</ul>\n" for 1 .. $level + 1;
 }
 
 sub _read_file {
